use strict;
use warnings;
use Test::More;

# generated by Dist::Zilla::Plugin::Test::PodSpelling 2.007004
use Test::Spelling 0.12;
use Pod::Wordlist;


add_stopwords(<DATA>);
all_pod_files_spelling_ok( qw( bin lib ) );
__DATA__
ABCDEFGHIJKLMNOPQRSTUVWXYZ
AWS
AnnoCPAN
Api
CLI
CPAN
Cache
Cas
CharacterGroup
CharacterGroupNotDefinedIronPolicyException
Client
ClientBase
Common
Connection
Connector
ConnectorBase
Github
HTTPS
IO
Iron
IronCache
IronCaches
IronHTTPCallException
IronIO
IronMQ
IronPolicyException
IronWorker
Ironcache
Item
JSON
Koivunalho
MERCHANTABILITY
Message
Mikko
NoIronPolicyException
OAuth
POSIX
Params
Policy
PolicyBase
Queue
RESTful
Rackspace
SaaS
Storable
Subdirectory
TODO
Task
Timestamp
YAML
abcdefghijklmnopqrstuvwxyz
alnum
ascii
aws
cas
dir
filename
http
https
io
ironcache
ironmq
ironworker
json
lib
licensable
lv
mikko
msg
multi
params
runtime
scalability
semafores
startup
subitem
subparam
succcessful
successfull
timestamp
tradename
unreserves
url
webhooks
webservices
wildcard
wildcards
